/////////////////////////////////////////////////////////////////////////////
//
// Project:         RemoSens
//
// Name:            consoleContAv.cpp
//
// Purpose:         Programming example on how to communicate with the
//                  ethernet/network interface-box 'RemoSens', controlling
//                  a Rohde&Schwarz NRP-Zxx series USB power sensor
//
// Description:
//
// Author:          Juergen D. Geltinger
// Created:         2018-06-22
// Modified by:     Juergen D. Geltinger
// Modified:        2020-10-25
//
// Support:         support@ifd.de
//
// Acknowledgement: 'Rohde&Schwarz' and 'NRP' are registred trademarks
//                  of Rohde & Schwarz GmbH & Co KG, Munich
//
// Copyright:       IFD Ing.-Buero fuer Datentechnik, Altdorf
//
/////////////////////////////////////////////////////////////////////////////


#include <windows.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>

// Should have an installed (R&S-)VISA
#include "visa.h"

// Our VISA is expected to be capable of communicating with HiSLIP protocol...
// ...otherwise SCPI-RAW socket protocol could be used
#define HAVE_VISA_HISLIP 1


// Definition of Statusbits
const int nZeroCalBit    = 1 << 0;
const int nDevStatusBit  = 1 << 1;
const int nErrQueueBit   = 1 << 2;
const int nStatQuestBit  = 1 << 3;
const int nMessAvailBit  = 1 << 4;
const int nEvtStatRegBit = 1 << 5;
const int nServiceReqBit = 1 << 6;
const int nStatOperBit   = 1 << 7;

// -----------------------------------------------------------------
int CharFreq(const char *cpIn, char x = ',')
{
  int iRet = 0;
  while (*cpIn)
  {
    if (*cpIn++ == x)
      iRet++;
  }
  return iRet;
}


// -----------------------------------------------------------------
// Helper function: Wrapper for VISA viWrite()
// -----------------------------------------------------------------
ViStatus WriteToDevice(ViSession sess, const char *cpCmd)
{
  if (!cpCmd)
    return -1;

  char szCmd[1024];
  ViStatus lStat = VI_SUCCESS;
  ViUInt32 nCnt = 0;
  int  iCmdLen = strlen(cpCmd);

  if (iCmdLen < 1)
    return -2;

  memset(szCmd, 0x00, sizeof(szCmd));
  strncpy(szCmd, cpCmd, sizeof(szCmd)-1);

  if (szCmd[iCmdLen - 1] != '\n')
  {
    strcat(szCmd, "\n");
    iCmdLen++;
  }

  lStat = viWrite(sess, (ViBuf)szCmd, iCmdLen, &nCnt);
//printf("lStat = %ld, nCnt = %d\n", lStat, nCnt);

  return lStat;
}


// -----------------------------------------------------------------
// Helper function: Wrapper for VISA viWrite() & viRead()
// -----------------------------------------------------------------
ViStatus QueryFromDevice(ViSession sess, const char *cpCmd, char *cpResult, int iMaxCnt)
{
  if (!cpCmd)
    return -1;

  if (!cpResult)
    return -2;

  if (iMaxCnt < 1)
    return -3;


  ViStatus lStat = VI_SUCCESS;
  ViUInt32 nCnt = 0;

  if (VI_SUCCESS == WriteToDevice(sess, cpCmd))
  {
    memset(cpResult, 0x00, iMaxCnt);

    lStat = viRead(sess, (ViBuf)cpResult, iMaxCnt, &nCnt);
//  printf("lStat = %ld, nCnt = %d\n", lStat, nCnt);
  }

  return lStat;
}


// -----------------------------------------------------------------
// Zero Calibration
// -----------------------------------------------------------------
void DoZeroCalibration(ViSession sess)
{
  char szStat[80];
  int  iCalibrationStat = 0;

  WriteToDevice(sess, "CAL:ZERO:AUTO ONCE");

  for (int n = 0; n < 50; n++)
  {
    QueryFromDevice(sess, "STAT:OPER:COND?", szStat, sizeof(szStat));
    iCalibrationStat = atoi(szStat);
    if (iCalibrationStat & nZeroCalBit)
      printf("Zero Calibration in progress...\n");
    else
      break;

    Sleep(250);
  }
}


// -----------------------------------------------------------------
// main
// -----------------------------------------------------------------
int main()
{
  ViSession rm;
  ViSession session;
  char szBuf[300000];
  long lStat = 0;


  // Opening a VISA HiSLIP connection to a 'RemoSens' interface-box
  // Hint: 'RemoSens' displays its TCP/IP address on the front panel;
  //       replace the following IP address with the local one of
  //       your 'RemoSens'
  viOpenDefaultRM(&rm);
  
#ifdef HAVE_VISA_HISLIP
  viOpen(rm, "TCPIP::192.168.2.108::HISLIP", 0, 10000, &session);
#else
  viOpen(rm, "TCPIP::192.168.2.108::5025::socket", 0, 10000, &session);
  viSetAttribute(session, VI_ATTR_TERMCHAR, 0x0a);  // This is important...
  viSetAttribute(session, VI_ATTR_TERMCHAR_EN, 1);  // ...when using SCPI-RAW
#endif


  // Information about RemoSens Interface-Box
  // -----------------------------------------------------------------
  QueryFromDevice(session, "SYST:IFVersion?", szBuf, sizeof(szBuf));
  printf("'RemoSens' Interface-Box Version: %s", szBuf);
  // -----------------------------------------------------------------


  // Query the Identification of the connected power sensor
  // -----------------------------------------------------------------
  QueryFromDevice(session, "*IDN?", szBuf, sizeof(szBuf));
  printf("*IDN? = %s", szBuf);
  // -----------------------------------------------------------------


  // Select the so called 'Continuous Average Mode'
  // -----------------------------------------------------------------
  WriteToDevice(session, "SENS:FUNC \"POW:AVG\"");


  // Set measurement frequency (here 2.4 GHz)
  // -----------------------------------------------------------------
  WriteToDevice(session, "SENS:FREQ 2.4e9");


  // Select immediate Trigger (= not waiting for a signal edge)
  // -----------------------------------------------------------------
  WriteToDevice(session, "TRIG:SOUR IMM");


  // Select a measurement window (aperture) of 10ms
  // -----------------------------------------------------------------
  WriteToDevice(session, "SENS:POW:AVG:APER 10e-3");


  // Select an average count of 4,
  // deactivate AUTO-Averaging, activate manual averaging
  // and repeating average (see Power Sensor User Manuals
  // for detailed informations)
  // -----------------------------------------------------------------
  WriteToDevice(session, "SENS:AVER:COUN:AUTO OFF");
  WriteToDevice(session, "SENS:AVER:COUN 4");
  WriteToDevice(session, "SENS:AVER:TCON REP");
  WriteToDevice(session, "SENS:AVER:STAT ON");


  // We want dBm results
  // -----------------------------------------------------------------
  WriteToDevice(session, "UNIT:POW DBM");


  // Zero Calibration
  // -----------------------------------------------------------------
  printf("Switch OFF any RF input power to the sensor for Zero Calibration. Then press ENTER...");
  getchar();
  printf("\n");

  DoZeroCalibration(session);

  printf("Switch ON RF input power to the sensor. Then press ENTER for measurement start...");
  getchar();
  printf("\n");


  // Now execute some measurements. The resulting data array
  // should consist of the specified number of Trace Points
  // -----------------------------------------------------------------
  for (int nLoop = 0; nLoop < 10; nLoop++)
  {
    bool bMeasIsReady = false;
    ViUInt16 uiSTB = 0;

    // Statusbyte before measurement start
    QueryFromDevice(session, "*STB?", szBuf, sizeof(szBuf));
    printf("*STB? = %s\n", szBuf);

    // Measurement start
    WriteToDevice(session, "INIT:IMM");


    // Poll for the end of each Trace measurement
    int nMaxPolling = 40;   // with the Sleep(50) call below
                            // this gives a max polling time of approx. 2000ms

    for (int n = 0; (n < nMaxPolling) && !bMeasIsReady; n++)
    {
      // If MeasReady-Bit is set, the measurement is finished
      QueryFromDevice(session, "*STB?", szBuf, sizeof(szBuf));
      printf("*STB? = %s\n", szBuf);

      uiSTB = atoi(szBuf);
      if ((uiSTB & nMessAvailBit) == nMessAvailBit)
        bMeasIsReady = true;
      else
        Sleep(50);
    }

    if (bMeasIsReady)
    {
      QueryFromDevice(session, "FETCH?", szBuf, sizeof(szBuf));

      // Print the results...
      printf("FETCH? = %s  ", szBuf);
      printf("               HINT: 9.910000e+37 is defined as SCPI NaN (not-a-number) = unmeasured Min/Max values\n");

      double dPowerInDBM = atof(szBuf);
      printf("Power = %.3f dBm\n\n", dPowerInDBM);
    }
  }

  viClose(session);
  viClose(rm);

  printf("\nPress <ENTER> to finish the application...");
  getchar();
  printf("\n");
  return 0;
}

